/*
 * rlocate_updatedb.c
 *
 * Copyright (C) 2004 Rasto Levrinc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * rlocate_updatedb moves the rlocatedb to rlocatedb_temp, calls the original 
 * updatedb and after that it removes the rlocatedb_temp.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <string.h>

#define RLOCATEDB "/var/cache/locate/rlocatedb"
#define RLOCATEDB_TEMP "/var/cache/locate/rlocatedb_temp"
#define UPDATEDB "updatedb"

static int fd_db;

/*
 * main
 */
int main(argc, argv) 
        int argc;
        char *argv[];
{
        int moved = 0;
        /* rename rlocatedb to rlocatedb_temp. If it's locked wait. */
        if ( (fd_db = open(RLOCATEDB, O_WRONLY)) < 0 )
        {
                fprintf(stderr, "%s: can't open "RLOCATEDB": %s\n", 
                                argv[0], strerror(errno) );
        } else {
                struct flock fl = { F_WRLCK, SEEK_SET, 0, 0, 0 };
                fl.l_pid = getpid();
                if (fcntl(fd_db, F_SETLKW, &fl) < 0) {
                        fprintf(stderr, "%s: can't get a lock on "RLOCATEDB
                                        ": %s\n", argv[0], strerror(errno) );
                } else {
                /* rename releases the lock */
                        if(rename(RLOCATEDB, RLOCATEDB_TEMP)<0) 
                                fprintf(stderr, "%s: can't rename "RLOCATEDB
                                                " to "RLOCATEDB_TEMP": %s\n", 
                                                argv[0], strerror(errno) );
                        else
                                moved = 1;
                }
                if (close(fd_db) <0)
                        fprintf(stderr, "%s: can't close "RLOCATEDB": %s\n", 
                                        argv[0], strerror(errno) );
        }

        /* call original updatedb */
        pid_t pid;
        if ( (pid = fork())<0)
                fprintf(stderr, "%s: fork error: %s\n", 
                                argv[0], strerror(errno) );
        else if (pid == 0) {
                if (execvp(UPDATEDB, argv) < 0)
                        fprintf(stderr, "%s: execvp error: %s\n", 
                                        argv[0], strerror(errno) );
        }
        if (waitpid(pid, NULL, 0) < 0)
                fprintf(stderr, "%s: waitpid error: %s\n", 
                                argv[0], strerror(errno) );
        /* remove rlocatedb_temp */
        if (moved == 1) {
                if (unlink(RLOCATEDB_TEMP) < 0)
                        fprintf(stderr, "%s: can't unlink "RLOCATEDB": %s\n", 
                                        argv[0], strerror(errno) );
        }
        return(0);
}


